@@ -121,13 +121,13 @@ static field_props field[MAXFIELDSIZE * MAXFIELDSIZE];
 /* Pre-rendering image data prepared from file. */
 static GamesPreimage *ball_preimage = NULL;
 /* The tile images with balls rendered on them. */
-static GdkPixmap *ball_pixmap = NULL;
+static cairo_surface_t *ball_surface = NULL;
 /* The balls rendered to a size appropriate for the preview. */
-static GdkPixmap *preview_pixmaps[7] = { NULL, NULL, NULL, NULL, NULL, NULL, NULL };
+static cairo_surface_t *preview_surfaces[7] = { NULL, NULL, NULL, NULL, NULL, NULL, NULL };
 
-/* A pixmap of a blank tile. */
-static GdkPixmap *blank_pixmap = NULL;
-static GdkPixmap *blank_preview_pixmap = NULL;
+/* A cairo_surface_t of a blank tile. */
+static cairo_surface_t *blank_surface = NULL;
+static cairo_surface_t *blank_preview_surface = NULL;
 
 static GtkWidget *fast_moves_toggle_button = NULL;
 
@@ -268,7 +268,7 @@ refresh_pixmaps (void)
   /* Since we get called both by configure and after loading an image.
    * it is possible the pixmaps aren't initialised. If they aren't
    * we don't do anything. */
-  if (!ball_pixmap)
+  if (!ball_surface)
     return;
 
   if (ball_preimage) {
@@ -298,7 +298,7 @@ refresh_pixmaps (void)
   g_free (warning_message);
   warning_message = NULL;
 
-  cr = gdk_cairo_create (ball_pixmap);
+  cr = cairo_create (ball_surface);
   gdk_cairo_set_source_color (cr, &backgnd.color);
 
   cairo_rectangle (cr, 0, 0, boxsize * 4, boxsize * 7);
@@ -307,7 +307,7 @@ refresh_pixmaps (void)
   gdk_cairo_set_source_pixbuf (cr, ball_pixbuf, 0, 0);
   cairo_mask (cr, cairo_get_source (cr));
 
-  cr_blank = gdk_cairo_create (blank_pixmap);
+  cr_blank = cairo_create (blank_surface);
   gdk_cairo_set_source_color (cr_blank, &backgnd.color);
 
   cairo_rectangle (cr_blank, 0, 0, boxsize, boxsize);
@@ -319,7 +319,7 @@ refresh_pixmaps (void)
 }
 
 static void
-refresh_preview_pixmaps (void)
+refresh_preview_surfaces (void)
 {
   guint i;
   GdkPixbuf *scaled = NULL;
@@ -342,8 +342,8 @@ refresh_preview_pixmaps (void)
    * This code assumes that each preview window is identical. */
 
   for (i = 0; i < 7; i++)
-    if (preview_pixmaps[i])
-      g_object_unref (preview_pixmaps[i]);
+    if (preview_surfaces[i])
+      cairo_surface_destroy (preview_surfaces[i]);
 
   if (ball_preimage)
     scaled = games_preimage_render (ball_preimage, 4 * preview_width,
@@ -356,9 +356,10 @@ refresh_preview_pixmaps (void)
   }
 
   for (i = 0; i < 7; i++) {
-    preview_pixmaps[i] = gdk_pixmap_new (gtk_widget_get_window (widget),
-                                         preview_width, preview_height, -1);
-    cr = gdk_cairo_create (preview_pixmaps[i]);
+    preview_surfaces[i] = gdk_window_create_similar_surface (gtk_widget_get_window (widget),
+                                                             CAIRO_CONTENT_COLOR_ALPHA,
+                                                             preview_width, preview_height);
+    cr = cairo_create (preview_surfaces[i]);
     gdk_cairo_set_source_color (cr, &gtk_widget_get_style (widget)->bg[GTK_STATE_NORMAL]);
     gdk_cairo_rectangle (cr, &preview_rect);
     cairo_fill (cr);
@@ -369,12 +370,13 @@ refresh_preview_pixmaps (void)
     cairo_destroy (cr);
   }
 
-  if (blank_preview_pixmap)
-    g_object_unref (blank_preview_pixmap);
+  if (blank_preview_surface)
+    cairo_surface_destroy (blank_preview_surface);
 
-  blank_preview_pixmap = gdk_pixmap_new (gtk_widget_get_window (widget), 
-                                         preview_width, preview_height, -1);
-  cr = gdk_cairo_create (blank_preview_pixmap);
+  blank_preview_surface = gdk_window_create_similar_surface (gtk_widget_get_window (widget),
+                                                             CAIRO_CONTENT_COLOR_ALPHA,
+                                                             preview_width, preview_height);
+  cr = cairo_create (blank_preview_surface);
   gdk_cairo_set_source_color (cr, &gtk_widget_get_style (widget)->bg[GTK_STATE_NORMAL]);
   gdk_cairo_rectangle (cr, &preview_rect);
   cairo_fill (cr);
@@ -473,19 +475,20 @@ void
 draw_preview (void)
 {
   guint i;
+  cairo_pattern_t *pattern;
 
   for (i = 0; i < MAXNPIECES; i++) {
 
     if (i < npieces)
-      gdk_window_set_back_pixmap (gtk_widget_get_window (preview_widgets[i]),
-                                  preview_pixmaps[preview[i] - 1], FALSE);
+      pattern = cairo_pattern_create_for_surface (preview_surfaces[preview[i] - 1]);
     else
-      gdk_window_set_back_pixmap (gtk_widget_get_window (preview_widgets[i]),
-                                  blank_preview_pixmap, FALSE);
+      pattern = cairo_pattern_create_for_surface (blank_preview_surface);
 
-    gdk_window_clear (gtk_widget_get_window (preview_widgets[i]));
+    gdk_window_set_background_pattern (gtk_widget_get_window (preview_widgets[i]),
+                                       pattern);
+    cairo_pattern_destroy (pattern);
+    gtk_widget_queue_draw (preview_widgets[i]);
   }
-
 }
 
 static void
@@ -519,11 +522,9 @@ static void
 game_over (void)
 {
   int pos;
-  GamesScoreValue hiscore;
 
   set_statusbar_message (_("Game Over!"));
-  hiscore.plain = score;
-  pos = games_scores_add_score (highscores, hiscore);
+  pos = games_scores_add_plain_score (highscores, score);
   show_scores (pos, TRUE);
   return;
 }
@@ -783,42 +784,42 @@ key_press_event (GtkWidget * widget, GdkEventKey * event, void *d)
   key = event->keyval;
 
   switch (key) {
-  case GDK_Left:
-  case GDK_KP_Left:
+  case GDK_KEY_Left:
+  case GDK_KEY_KP_Left:
     move_cursor (-1, 0);
     break;
-  case GDK_Right:
-  case GDK_KP_Right:
+  case GDK_KEY_Right:
+  case GDK_KEY_KP_Right:
     move_cursor (1, 0);
     break;
-  case GDK_Up:
-  case GDK_KP_Up:
+  case GDK_KEY_Up:
+  case GDK_KEY_KP_Up:
     move_cursor (0, -1);
     break;
-  case GDK_Down:
-  case GDK_KP_Down:
+  case GDK_KEY_Down:
+  case GDK_KEY_KP_Down:
     move_cursor (0, 1);
     break;
-  case GDK_Home:
-  case GDK_KP_Home:
+  case GDK_KEY_Home:
+  case GDK_KEY_KP_Home:
     move_cursor (-999, 0);
     break;
-  case GDK_End:
-  case GDK_KP_End:
+  case GDK_KEY_End:
+  case GDK_KEY_KP_End:
     move_cursor (999, 0);
     break;
-  case GDK_Page_Up:
-  case GDK_KP_Page_Up:
+  case GDK_KEY_Page_Up:
+  case GDK_KEY_KP_Page_Up:
     move_cursor (0, -999);
     break;
-  case GDK_Page_Down:
-  case GDK_KP_Page_Down:
+  case GDK_KEY_Page_Down:
+  case GDK_KEY_KP_Page_Down:
     move_cursor (0, 999);
     break;
 
-  case GDK_space:
-  case GDK_Return:
-  case GDK_KP_Enter:
+  case GDK_KEY_space:
+  case GDK_KEY_Return:
+  case GDK_KEY_KP_Enter:
     if (show_cursor)
       cell_clicked (widget, cursor_x, cursor_y);
     break;
@@ -828,19 +829,15 @@ key_press_event (GtkWidget * widget, GdkEventKey * event, void *d)
 }
 
 static void
-draw_grid (void)
+draw_grid (cairo_t *cr)
 {
-  cairo_t *cr;
   GdkColor color;
   guint w, h;
   guint i;
-  GtkAllocation allocation;
 
-  gtk_widget_get_allocation (draw_area, &allocation);
-  w = allocation.width;
-  h = allocation.height;
+  w = gtk_widget_get_allocated_width(draw_area);
+  h = gtk_widget_get_allocated_height(draw_area);
 
-  cr = gdk_cairo_create (gtk_widget_get_window (draw_area));
   gdk_color_parse ("#525F6C", &color);
   gdk_cairo_set_source_color (cr, &color);
   cairo_set_line_width (cr, 1.0);
@@ -859,28 +856,16 @@ draw_grid (void)
 
   cairo_rectangle (cr, 0.5, 0.5, w - 0.5, h - 0.5);
   cairo_stroke (cr);
-
-  cairo_destroy (cr);
 }
 
-/* Redraw a part of the field */
 static gboolean
-field_expose_event (GtkWidget * widget, GdkEventExpose * event, gpointer gp)
+field_draw_callback (GtkWidget * widget, cairo_t *cr)
 {
-  GdkWindow *window = gtk_widget_get_window (widget);
-  cairo_t *cr;
-  guint x_start, x_end, y_start, y_end, i, j, idx;
+  guint i, j, idx;
+  GdkColor cursorColor;
 
-  x_start = event->area.x / boxsize;
-  x_end = (event->area.x + event->area.width - 1) / boxsize + 1;
-
-  y_start = event->area.y / boxsize;
-  y_end = (event->area.y + event->area.height - 1) / boxsize + 1;
-
-  cr = gdk_cairo_create (window);
-
-  for (i = y_start; i < y_end; i++) {
-    for (j = x_start; j < x_end; j++) {
+  for (i = 0; i < vfieldsize; i++) {
+    for (j = 0; j < hfieldsize; j++) {
       int phase, color;
 
       idx = j + i * hfieldsize;
@@ -893,38 +878,32 @@ field_expose_event (GtkWidget * widget, GdkEventExpose * event, gpointer gp)
 
         phase = ABS (ABS (3 - phase) - 3);
 
-        gdk_cairo_set_source_pixmap (cr, ball_pixmap,
-                                     (1.0 * j - phase) * boxsize,
-                                     (1.0 * i - color) * boxsize);
+        cairo_set_source_surface (cr, ball_surface,
+                                 (1.0 * j - phase) * boxsize,
+                                 (1.0 * i - color) * boxsize);
       } else {
-        gdk_cairo_set_source_pixmap (cr, blank_pixmap, 0, 0);
+        cairo_set_source_surface (cr, blank_surface, 1.0 * j * boxsize, 1.0 * i * boxsize);
       }
 
       cairo_fill (cr);
     }
   }
 
-  if (show_cursor && cursor_x >= x_start && cursor_x <= x_end
-      && cursor_y >= y_start && cursor_y <= y_end) {
-    GdkColor color;
+  /* Cursor */
+  if (((backgnd.color.red + backgnd.color.green + backgnd.color.blue) / 3) >
+      (G_MAXUINT16 / 2))
+    gdk_color_parse ("#000000", &cursorColor);
+  else
+    gdk_color_parse ("#FFFFFF", &cursorColor);
 
-    if (((backgnd.color.red + backgnd.color.green + backgnd.color.blue) / 3) >
-        (G_MAXUINT16 / 2))
-      gdk_color_parse ("#000000", &color);
-    else
-      gdk_color_parse ("#FFFFFF", &color);
+  gdk_cairo_set_source_color (cr, &cursorColor);
+  cairo_set_line_width (cr, 1.0);
+  cairo_rectangle (cr,
+                   cursor_x * boxsize + 1.5, cursor_y * boxsize + 1.5,
+                   boxsize - 2.5, boxsize - 2.5);
+  cairo_stroke (cr);
 
-    gdk_cairo_set_source_color (cr, &color);
-    cairo_set_line_width (cr, 1.0);
-    cairo_rectangle (cr,
-                     cursor_x * boxsize + 1.5, cursor_y * boxsize + 1.5,
-                     boxsize - 2.5, boxsize - 2.5);
-    cairo_stroke (cr);
-  }
-
-  cairo_destroy (cr);
-
-  draw_grid ();
+  draw_grid (cr);
 
   return FALSE;
 }
@@ -1226,11 +1205,7 @@ game_about_callback (GtkAction * action, gpointer * data)
   gchar *license = games_get_license (_("Five or More"));
 
   gtk_show_about_dialog (GTK_WINDOW (app),
-#if GTK_CHECK_VERSION (2, 11, 0)
                          "program-name", _("Five or More"),
-#else
-                         "name", _("Five or More"),
-#endif
                          "version", VERSION,
                          "comments", _("GNOME port of the once-popular Color Lines game.\n\nFive or More is a part of GNOME Games."),
                          "copyright",
@@ -1249,9 +1224,6 @@ game_about_callback (GtkAction * action, gpointer * data)
 static void
 set_backgnd_color (const gchar * str)
 {
-  GdkColormap *colormap;
-  GtkStyle *widget_style, *temp_style;
-
   if (!str)
     str = g_strdup ("#000000");
 
@@ -1263,17 +1235,6 @@ set_backgnd_color (const gchar * str)
   if (!gdk_color_parse (backgnd.name, &backgnd.color)) {
     gdk_color_parse ("#000000", &backgnd.color);
   }
-
-  colormap = gtk_widget_get_colormap (draw_area);
-  gdk_colormap_alloc_color (colormap, &backgnd.color, FALSE, TRUE);
-  widget_style = gtk_widget_get_style (draw_area);
-  temp_style = gtk_style_copy (widget_style);
-  temp_style->bg[0] = backgnd.color;
-  temp_style->bg[1] = backgnd.color;
-  temp_style->bg[2] = backgnd.color;
-  temp_style->bg[3] = backgnd.color;
-  temp_style->bg[4] = backgnd.color;
-  gtk_widget_set_style (draw_area, temp_style);
 }
 
 static void
@@ -1301,7 +1262,7 @@ load_theme (void)
   ball_preimage = load_image (ball_filename);
 
   refresh_pixmaps ();
-  refresh_preview_pixmaps ();
+  refresh_preview_surfaces ();
 }
 
 static void
@@ -1438,7 +1399,6 @@ game_props_callback (void)
                                                GTK_DIALOG_DESTROY_WITH_PARENT,
                                                GTK_STOCK_CLOSE,
                                                GTK_RESPONSE_CLOSE, NULL);
-    gtk_dialog_set_has_separator (GTK_DIALOG (pref_dialog), FALSE);
     g_signal_connect (pref_dialog, "response",
                       G_CALLBACK (pref_dialog_response), NULL);
     g_signal_connect (pref_dialog, "delete-event",
@@ -1551,7 +1511,7 @@ preview_configure_cb (GtkWidget * widget, GdkEventConfigure * event)
   preview_width = event->width;
   preview_height = event->height;
 
-  refresh_preview_pixmaps ();
+  refresh_preview_surfaces ();
 
   draw_preview ();
 
@@ -1561,17 +1521,19 @@ preview_configure_cb (GtkWidget * widget, GdkEventConfigure * event)
 static int
 configure_event_callback (GtkWidget * widget, GdkEventConfigure * event)
 {
-  if (ball_pixmap)
-    g_object_unref (ball_pixmap);
+  if (ball_surface)
+    cairo_surface_destroy(ball_surface);
 
-  if (blank_pixmap)
-    g_object_unref (blank_pixmap);
+  if (blank_surface)
+    cairo_surface_destroy (blank_surface);
 
   boxsize = (event->width - 1) / hfieldsize;
-  ball_pixmap = gdk_pixmap_new (gtk_widget_get_window (draw_area), boxsize * 4, 
-                                boxsize * 7, -1);
-  blank_pixmap = gdk_pixmap_new (gtk_widget_get_window (draw_area), boxsize, 
-                                 boxsize, -1);
+  ball_surface = gdk_window_create_similar_surface (gtk_widget_get_window (draw_area),
+                                                    CAIRO_CONTENT_COLOR_ALPHA,
+                                                    boxsize * 4, boxsize * 7);
+  blank_surface = gdk_window_create_similar_surface (gtk_widget_get_window (draw_area),
+                                                     CAIRO_CONTENT_COLOR_ALPHA,
+                                                     boxsize, boxsize);
   refresh_pixmaps ();
 
   refresh_screen ();
@@ -1623,26 +1585,6 @@ quit_cb (EggSMClient *client,
   return FALSE;
 }
 
-#endif /* WITH_SMCLIENT */
-
-static void
-load_properties (void)
-{
-  gchar *buf;
-
-  ball_filename = games_conf_get_string_with_default (KEY_PREFERENCES_GROUP, KEY_BALL_THEME, DEFAULT_BALL_THEME);
-
-  move_timeout = games_conf_get_integer (KEY_PREFERENCES_GROUP, KEY_MOVE_TIMEOUT, NULL);
-  if (move_timeout <= 0)
-    move_timeout = 100;
-
-  buf = games_conf_get_string_with_default (KEY_PREFERENCES_GROUP, KEY_BACKGROUND_COLOR, "#000000"); /* FIXMEchpe? */
-  set_backgnd_color (buf);
-  g_free (buf);
-
-  load_theme ();
-}
-
 static void
 restart (void)
 {
@@ -1674,6 +1616,26 @@ restart (void)
   }
 }
 
+#endif /* WITH_SMCLIENT */
+
+static void
+load_properties (void)
+{
+  gchar *buf;
+
+  ball_filename = games_conf_get_string_with_default (KEY_PREFERENCES_GROUP, KEY_BALL_THEME, DEFAULT_BALL_THEME);
+
+  move_timeout = games_conf_get_integer (KEY_PREFERENCES_GROUP, KEY_MOVE_TIMEOUT, NULL);
+  if (move_timeout <= 0)
+    move_timeout = 100;
+
+  buf = games_conf_get_string_with_default (KEY_PREFERENCES_GROUP, KEY_BACKGROUND_COLOR, "#000000"); /* FIXMEchpe? */
+  set_backgnd_color (buf);
+  g_free (buf);
+
+  load_theme ();
+}
+
 static const GtkActionEntry actions[] = {
   {"GameMenu", NULL, N_("_Game")},
   {"SettingsMenu", NULL, N_("_Settings")},
@@ -1773,10 +1735,7 @@ main (int argc, char *argv[])
   rgen = g_rand_new ();
 
   context = g_option_context_new (NULL);
-#if GLIB_CHECK_VERSION (2, 12, 0)
   g_option_context_set_translation_domain (context, GETTEXT_PACKAGE);
-#endif
-
   g_option_context_add_group (context, gtk_get_option_group (TRUE));
 #ifdef WITH_SMCLIENT
   g_option_context_add_group (context, egg_sm_client_get_option_group ());
@@ -1833,7 +1792,7 @@ main (int argc, char *argv[])
   ui_manager = gtk_ui_manager_new ();
 
   games_stock_prepare_for_statusbar_tooltips (ui_manager, statusbar);
-  gtk_statusbar_set_has_resize_grip (GTK_STATUSBAR (statusbar), FALSE);
+  gtk_window_set_has_resize_grip (GTK_WINDOW (app), TRUE);
 
   vbox = gtk_vbox_new (FALSE, 0);
   gtk_container_add (GTK_CONTAINER (app), vbox);
@@ -1892,8 +1851,8 @@ main (int argc, char *argv[])
                     G_CALLBACK (key_press_event), NULL);
   g_signal_connect (draw_area, "configure-event",
                     G_CALLBACK (configure_event_callback), NULL);
-  g_signal_connect (draw_area, "expose-event",
-                    G_CALLBACK (field_expose_event), NULL);
+  g_signal_connect (draw_area, "draw",
+                    G_CALLBACK (field_draw_callback), NULL);
   gridframe = games_grid_frame_new (hfieldsize, vfieldsize);
   games_grid_frame_set_padding (GAMES_GRID_FRAME (gridframe), 1, 1);
   gtk_container_add (GTK_CONTAINER (gridframe), draw_area);
