@@ -166,9 +166,12 @@ static void
 gnibbles_worm_add_actor (GnibblesWorm *worm)
 {
   ClutterActor *actor;
+  GError *error = NULL;
 
-  actor = gtk_clutter_texture_new_from_pixbuf (
-            worm_pixmaps[properties->wormprops[worm->number]->color - 12]);
+  actor = gtk_clutter_texture_new ();
+  gtk_clutter_texture_set_from_pixbuf (GTK_CLUTTER_TEXTURE (actor),
+                                       worm_pixmaps[properties->wormprops[worm->number]->color - 12],
+                                       &error);
   clutter_actor_set_size (actor, properties->tilesize, properties->tilesize);
   clutter_actor_set_position (actor,
                               worm->xhead * properties->tilesize,
@@ -481,13 +484,16 @@ gnibbles_worm_animate_death (GnibblesWorm *worm)
 {
   ClutterActor *group = clutter_group_new ();
   ClutterActor *tmp = NULL;
+  GError *error = NULL;
 
   int i;
   gfloat x,y;
 
   for (i = 0; i < g_list_length (worm->list); i++) {
-    tmp = gtk_clutter_texture_new_from_pixbuf (
-            worm_pixmaps [properties->wormprops[worm->number]->color - 12]);
+    tmp = gtk_clutter_texture_new ();
+    gtk_clutter_texture_set_from_pixbuf (GTK_CLUTTER_TEXTURE (tmp),
+                                         worm_pixmaps[properties->wormprops[worm->number]->color - 12],
+                                         &error);
 
     clutter_actor_get_position (CLUTTER_ACTOR (g_list_nth_data (worm->list, i)),
                                 &x, &y);
@@ -661,7 +667,7 @@ gnibbles_worm_rescale (GnibblesWorm *worm, gint tilesize)
                                 (y_pos / properties->tilesize) * tilesize);
 
     gtk_clutter_texture_set_from_pixbuf (
-       CLUTTER_TEXTURE (tmp),
+       GTK_CLUTTER_TEXTURE (tmp),
        worm_pixmaps[properties->wormprops[worm->number]->color - 12],
        &err);
     if (err)
@@ -704,6 +710,7 @@ gnibbles_worm_reduce_tail (GnibblesWorm *worm, gint erasesize)
   gfloat x,y;
   ClutterActor *tmp = NULL;
   ClutterActor *group = clutter_group_new ();
+  GError *error = NULL;
 
   if (erasesize) {
     if (g_list_length (worm->list) <= erasesize) {
@@ -712,8 +719,10 @@ gnibbles_worm_reduce_tail (GnibblesWorm *worm, gint erasesize)
     }
 
     for (i = 0; i < erasesize; i++) {
-      tmp = gtk_clutter_texture_new_from_pixbuf (
-              worm_pixmaps[properties->wormprops[worm->number]->color - 12]);
+      tmp = gtk_clutter_texture_new ();
+      gtk_clutter_texture_set_from_pixbuf (GTK_CLUTTER_TEXTURE (tmp),
+                                           worm_pixmaps[properties->wormprops[worm->number]->color - 12],
+                                           &error);
       clutter_actor_get_position
         (CLUTTER_ACTOR (g_list_last (worm->list)->data), &x, &y);
       clutter_actor_set_position (CLUTTER_ACTOR (tmp), x, y);
@@ -727,7 +736,7 @@ gnibbles_worm_reduce_tail (GnibblesWorm *worm, gint erasesize)
     worm->length -= erasesize;
     clutter_container_add_actor (CLUTTER_CONTAINER (stage), group);
 
-    clutter_actor_animate (group, CLUTTER_EASE_OUT_ELASTIC, 850,
+    clutter_actor_animate (group, CLUTTER_EASE_OUT_EXPO, 850,
                            "opacity", 0,
                            NULL);
   }
